########################################################################################
##                                                                                    ##
## FUNCTION:  CreateGeneTable                                                         ##
##                                                                                    ##
## THIS FUNCTION TAKES A GENE TABLE IN THE FORM OF A DATAFRAME, AND ANNOTATES         ##
## THE GENES IN THE GENE TABLE.  THE OUTPUT IS A SET OF HTML FILES CONTAINING THE     ##
## HYPERLINKED GENE TABLE AND THE ANNOTATIONS TABLE.  THE GENE ONTOLOGY ANALYSIS AND  ##
## CHROMOSOMAL DISTRIBUTION BARPLOT MAY ALSO BE RUN AS AN OPTION.  ALL OUTPUT FILES   ##
## ARE CREATED IN A USER-SPECIFIED FOLDER.  THE USER MAY RETURN THE HTML CODE IN      ##
## ORDER TO INSERT THE CODE INTO HIS OR HER OWN HTML FILE.  HOWEVER, THIS HTML CODE   ##
## SHOULD BE INSERTED INTO A FILE CONTAINED IN THE SAME FOLDER AS THE OUTPUT          ##
## ANNOTATIONS TABLE AND CHROMOSOMAL DISTRIBUTION BARPLOT FILES, IN ORDER FOR THE     ##
## HYPERLINKS IN THE GENE TABLE TO FUNCTION PROPERLY.                                 ##
##                                                                                    ##
########################################################################################
## 4/11/17  Ting removed affymetrix links for probesets.
## 3/16/15	Lori added code for hypertext Ensembl ID.
########################################################
CreateGeneTable <- function(
  GeneTable
  , AllGenesInAnalysis=NULL
  , InsertAfterColumn=1
  , RunChromDistBarPlot=T
  , RunGOOvE=T
  , MinObserved=5
  , MinRatio=2
  , RedoExpectedTable=T
  , FolderPath=""
  , ReturnHtmlCode=T
  , GeneTableFile=""
  , AnnotationsTableFile=""
  , DoMasterAnnotations=T
  , ChromDistBarPlotFile=""
  , HeaderStringsVec="Output of plugin analysis"
  , TableCaption=""
  , ColumnNum
  , GeneIdColumnToMatch=1
  , GeneRowNum=NULL
  , TableID=NULL
) {


## INPUT:
##
## (REQUIRED PARAMETERS:)
##   GeneTable          = data frame containing genes, where first column is the gene id used in first column of 'Gene identifiers' worksheet.
##                Columns with names matching specific keywords ('Clone', 'Probe set', 'GB acc', 'UG cluster', or 'Gene symbol')
##                will also be hyperlinked in the resulting HTML gene table.  Periods in columns names will be replaced by spaces before matching.
##   AllGenesInAnalysis     = REQUIRED ONLY IF RunGOOvE IS TRUE.  Vector of gene ids for ALL genes used in analysis.
##
## (OPTIONAL PARAMETERS:)
##   InsertAfterColumn      = number giving column after which annotations link will be inserted.
##   RunChromDistBarPlot    = Boolean denoting whether or not to run the chromosomal distribution barplot.
##   RunGOOvE           = Boolean denoting whether or not to run the Gene Ontology observed versus expected table.
##   MinObserved        = Used only if RunGOOvE is TRUE.  Minimum observed value required in order for GO term to be printed in table.
##   MinRatio           = Used only if RunGOOvE is TRUE.  Minimum observed to expected ratio required in order for GO term to be printed in table.
##   RedoExpectedTable  = Used only if RunGOOvE is TRUE.  If RedoExpectedTable is TRUE, then compute expected values from scratch, instead of using saved values from file.
##   ReturnHtmlCode     = Boolean denoting whether function should return the HTML code, or write the HTML output to GeneTableFile.
##                NOTE: The HTML code returned by this function must be written to an HTML file contained in the folder specified
##                by FolderPath.  Otherwise the links to AnnotationsTableFile and ChromDistBarPlotFile will be broken!
##   FolderPath         = string denoting path of folder in which all output files will be written.
##   GeneTableFile      = string denoting filename for user-provided gene table into which annotations link will be inserted.
##   AnnotationsTableFile   = string denoting filename for annotations table in a separate file.
##   DoMasterAnnotations    = Boolean denoting whether or not to create one master annotations table for multiple gene tables in the HTML output. Yong 8/21/06
##   ChromDistBarPlotFile   = string denoting filename for chromosomal distribution barplot, if RunChromDistBarPlot is TRUE.
##   HeaderStringsVec       = vector of strings which contain lines of text to be inserted into GeneTableFile before the actual gene table.
##   TableCaption       = string to use as a caption for gene table
##   ColumnNum          = number giving column of numerical values. So numerical values will not be wrapped in html output. It is assumed columns from ColumnNum to ncol(GeneTable) are all numeric.
##   GeneIdColumnToMatch    = Column number of GeneTable which contains the column to match (e.g., primary id).
##
## OUTPUT:
##
##   If ReturnHtmlCode is TRUE, then the returned value of the function is a list containing components named
##   GeneTableHtmlCode, GOTableHtmlCode, and ChromDistHtmlCode.  Each component is a vector of strings which can
##   be printed to an HTML file using the 'cat' function in R.  If RunGOOvE is FALSE, then the GOTableHtmlCode
##   component is an empty string ("").  Similarly, if RunChromDistBarPlot is FALSE, then the ChromDistHtmlCode
##   is an empty string ("").  The HTML code in the GeneTableHtmlCode component contains links to AnnotationsTableFile,
##   and the HTML code in the ChromDistHtmlCode contains a link to ChromDistBarPlotFile if RunChromDistBarPlot is TRUE.
##   These links are relative links, so it is important that the HTML code returned by this function is written to an
##   HTML file which is contained in the folder specified by the FolderPath parameter.
##
##   If ReturnHtmlCode is FALSE, then GeneTableFile will be created in the folder specified by FolderPath, and will
##   contain relative links to AnnotationsTableFile and ChromDistBarPlotFile (if RunChromDistBarPlot
##   is TRUE).
##

  #Set default path and filenames if user did not specify any.
  if (FolderPath=="") FolderPath <- paste(ProjectPath, "/Output/Plugins", sep="")
  if (GeneTableFile=="") GeneTableFile <- "GeneTableFile.html"
  if (AnnotationsTableFile=="") AnnotationsTableFile <- "Annotations.html"
  if (ChromDistBarPlotFile=="") ChromDistBarPlotFile <- "ChromDist.jpg"
  ArrayToolsPath <- path.package("GSEA")
  AnnotationsFileName <- AnnotationsTableFile
  ChromDistFileName <- ChromDistBarPlotFile

  GeneTableFile <- paste(FolderPath, GeneTableFile, sep="/")
  AnnotationsTableFile <- paste(FolderPath, AnnotationsTableFile, sep="/")
  ChromDistBarPlotFile <- paste(FolderPath, ChromDistBarPlotFile, sep="/")

  #Make sure the user did not specify duplicate filenames.
  if (GeneTableFile==AnnotationsTableFile) {
    ErrMsg <- "ERROR: GeneTableFile and AnnotationsTableFile have the same filename."
    require(tcltk)
    tkmessageBox(message=ErrMsg,icon="info",type="ok")
    cat(ErrMsg)
  }
  if (RunChromDistBarPlot & (GeneTableFile==ChromDistBarPlotFile)) {
    ErrMsg <- "ERROR: GeneTableFile and ChromDistBarPlotFile have the same filename."
    require(tcltk)
    tkmessageBox(message=ErrMsg,icon="info",type="ok")
    cat(ErrMsg)
  }
  if (RunChromDistBarPlot & (AnnotationsTableFile==ChromDistBarPlotFile)) {
    ErrMsg <- "ERROR: AnnotationsTableFile and ChromDistBarPlotFile have the same filename."
    require(tcltk)
    tkmessageBox(message=ErrMsg,icon="info",type="ok")
    cat(ErrMsg)
  }

  #Create FolderPath if it doesn't yet exist.
  CreatePath(FolderPath)

 # source.out<-try(source(paste(ArrayToolsPath,"/R/Annotations.R", sep=""))) #Yong added testing try-error 1/4/07
 # if (class(source.out)=='try-error') {
  #    require(tcltk)
  #    tkmessageBox(message=paste('Error occurred in Annotations.R:', geterrmessage(), sep='\n'),icon="info",type="ok")
  #    cat(geterrmessage()); stop()
 # }

  GeneId <- as.matrix(GeneTable)
  GeneIdsMatch <- c(as.character(GeneId[,GeneIdColumnToMatch]))

  ## (Need to check if annotations functions will automatically convert to character during matching??)
  ## Also need to test on case where GeneTable has only one row or one column.
  ##  ==> One col is OK, and one row with one col is OK, but one row with more than one col does not hyperlink.

  ColNames <- dimnames(GeneId)[[2]]
  ColNames <- gsub(pattern="\\.", replacement=" ", x=ColNames)
  NumGeneIds <- ncol(GeneId)

  ## TRY TO CREATE ANNOTATIONS:

  options(warn=2)  #Converts warnings to errors in try function.

  DoAnnotations <- F
  # Ting commented out the following on 7/18/2019
  # cat("", "Attempting to create annotations ...", "", sep="\n")
  # try.out <- try ( annotations <- CreateAnnotations(GeneIds=GeneIdsMatch, PathName=FolderPath, AnnotationsFileName=AnnotationsFileName, CallFromPlugins=T, DoMasterAnnotations=DoMasterAnnotations) ) # add DoMasterAnnotations, Yong 8/21/06
  # DoAnnotations <- F
  # if ("try.out" %in% ls()) {   #If "try.out" object exists.
  #   if (!is.null(class(try.out))) {  #Necessary only for R1.6.1, which evaluates class(NULL) and class(NA) as NULL whereas R1.7.0 evaluates it as "NULL" and "logical" respectively.
  #     if (class(try.out) != "try-error") {
  #       if (!is.null(annotations)) {
  #         DoAnnotations <- T
  #       }
  #     } else { #Yong added testing try-error 1/4/07
  #       require(tcltk)
  #       tkmessageBox(message=paste('Error occurred in Annotations.R:', geterrmessage(), sep='\n'),icon="info",type="ok")
  #       cat(geterrmessage());
  #       stop();
  #     }
  #   } else {  #In R1.6.1, class(try.out) evaluates as NULL even if function evaluates correctly and try.out is a valid non-null object.
  #     if ("annotations" %in% ls()) {
  #       if (!is.null(annotations)) {
  #         DoAnnotations <- T
  #       }
  #     }
  #   }
  #   rm(try.out)
  # }
  #
  # if (DoAnnotations) {
  #   NumRows <- nrow(GeneId)
  #   NumCols <- ncol(GeneId)
  #   ColNames <- dimnames(GeneId)[[2]]
  #   ColNames <- gsub(pattern="\\.", replacement=" ", x=ColNames)
  #
  #   if (InsertAfterColumn < 1) {
  #     cat("Error: InsertAfterColumn parameter must be >= 1.")
  #   } else {
  #     if (InsertAfterColumn > NumCols) {
  #       cat(paste("Error: InsertAfterColumn parameter must be <= ", NumCols, ".", sep=""))
  #     }
  #   }
  #
  #   if (!missing(ColumnNum)) ColumnNum <- ColumnNum + 1 # because of annotations column is added to GeneId
  #   if (InsertAfterColumn == NumCols) {  #INSERT ANNOTATIONS INTO LAST COLUMN.
  #     ColNames <- c(ColNames, "Anno-<BR>tations")
  #     if (NumRows > 1) {
  #       GeneId <- cbind(GeneId, annotations)
  #     } else {    #For some reason, cbind() doesn't work properly when there is only one row.
  #       GeneId <- c(GeneId, annotations)
  #       GeneId <- matrix(GeneId, nrow=1)
  #     }
  #     dimnames(GeneId)[[2]] <- ColNames
  #   } else {
  #     ColNames <- c(ColNames[1:InsertAfterColumn], "Anno-<BR>tations", ColNames[(InsertAfterColumn+1):NumCols])
  #     if (NumRows > 1) {
  #       GeneId <- cbind(GeneId[,1:InsertAfterColumn], annotations, GeneId[,(InsertAfterColumn+1):NumCols])
  #     } else {    #For some reason, cbind() doesn't work properly when there is only one row.
  #       GeneId <- c(GeneId[1:InsertAfterColumn], annotations, GeneId[(InsertAfterColumn+1):NumCols])
  #       GeneId <- matrix(GeneId, nrow=1)
  #     }
  #     dimnames(GeneId)[[2]] <- ColNames
  #   }
  #   NumCols <- NumCols+1
  #   NumGeneIds <- ncol(GeneId)
  # } else {
  #   cat("", "==> 'CreateAnnotations' function could not be run.", "", sep="\n")
  # }

  ## TRY TO CREATE CHROMOSOMAL DISTRIBUTION BARPLOT:

  if (RunChromDistBarPlot) {
    AnalysisName <- ""
    cat("", "Attempting to create chromosomal distribution barplot ...", "", sep="\n")
    try.out <- try ( ChromDistBarPlot(GeneIdsMatch, ProjectPath, AnalysisName, ChromDistBarPlotFile, AllGenesInAnalysis) )
    DoChromDistBarPlot <- F
    if ("try.out" %in% ls()) {   #If "try.out" object exists.
      if (!is.null(class(try.out))) {  #Necessary only for R1.6.1, which evaluates class(NULL) and class(NA) as NULL whereas R1.7.0 evaluates it as "NULL" and "logical" respectively.
        if (class(try.out) != "try-error") {
          if (file.access(ChromDistBarPlotFile) == 0) {
            DoChromDistBarPlot <- T
          }
        }
      } else {  #In R1.6.1, class(try.out) evaluates as NULL even if function evaluates correctly and try.out is a valid non-null object.
        if (file.access(ChromDistBarPlotFile) == 0) {
          DoChromDistBarPlot <- T
        }
      }
      rm(try.out)
    }
    if (!DoChromDistBarPlot) {
      cat("", "==> 'ChromDistBarPlot' function could not be run.", "", sep="\n")
    }
  }
  ## TRY TO RUN GENE ONTOLOGY ANALYSIS:
  if(RunGOOvE){
    cat("", "Attempting to run Gene Ontology analysis ...", "", sep="\n")
    try.out <- try ( OEhtml <- GOanalysis(GeneIdsMatch, ProjectPath, MinObserved, MinRatio) )
    DoGOanalysis <- F
    if ("try.out" %in% ls()) {   #If "try.out" object exists.
      if (!is.null(class(try.out))) {  #Necessary only for R1.6.1, which evaluates class(NULL) and class(NA) as NULL whereas R1.7.0 evaluates it as "NULL" and "logical" respectively.
        if (class(try.out) != "try-error") {
          DoGOanalysis <- T
          if (length(OEhtml)==1) {
            if (!is.na(OEhtml)) {
              if (OEhtml=="") {
                DoGOanalysis <- F
              }
            }
          }
        }
      } else {  #In R1.6.1, class(try.out) evaluates as NULL even if function evaluates correctly and try.out is a valid non-null object.
        if ("OEhtml" %in% ls()) {
          DoGOanalysis <- T
          if (length(OEhtml)==1) {
            if (!is.na(OEhtml)) {
              if (OEhtml=="") {
                DoGOanalysis <- F
              }
            }
          }
        }
      }
      rm(try.out)
    }
    if (!DoGOanalysis) {
      cat("", "==> 'GOanalysis' function could not be run.", "", sep="\n")
      OEhtml <- ""
    }
  }else{
    OEhtml <- ""
  }

  options(warn=0)  #Set back to default.
  cat("", "Preparing output tables ...", "", sep="\n")

  ## HYPERTEXT CLONE IDS.
  indic <- (ColNames=="CloneID")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
#    temp <- paste("<A HREF=\"http://nciarray.nci.nih.gov/cgi-bin/clone_report.cgi?CRITERIA=clone&PARAMETER=", ## NCBI retires clone DB https://ncbiinsights.ncbi.nlm.nih.gov/2019/05/01/clone-db-retirement/
 #   ifelse(regexpr(":",temp) > 0, temp, paste("IMAGE:", temp, sep="")), "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT PROBE SET IDS.
  indic <- (ColNames=="ProbeSet")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
    #temp <- ifelse(temp=="","",paste("<A HREF=\"https://www.affymetrix.com/LinkServlet?probeset=", temp, "\" TARGET=_blank>", temp, "</A>", sep=""))
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT GENBANK ACCESSION NUMBERS.
  indic <- (ColNames=="Accession")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
    temp <- paste("<A HREF=\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=Nucleotide&term=", temp, "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT MicroRNA ACCESSION NUMBERS.
  indic <- (ColNames=="MicroRNAID")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
    temp <- paste("<A HREF=\"http://microrna.sanger.ac.uk/cgi-bin/sequences/mirna_entry.pl?id=", temp, "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT Entrez ID.
  indic <- (ColNames=="EntrezID")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
    temp <- paste("<A HREF=\"http://www.ncbi.nlm.nih.gov/sites/entrez?db=gene&cmd=search&term=", temp, "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT UNIGENE CLUSTER IDS.
  indic <- (ColNames=="UGCluster")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
	## RETIREMENT NOTICE: https://ncbiinsights.ncbi.nlm.nih.gov/2019/07/30/the-unigene-web-pages-are-now-retired/
#    PeriodIndex <- regexpr("\\.",temp)
#    Org <- ifelse(PeriodIndex==-1, "", substring(temp, 1, PeriodIndex-1))
#    Number <- ifelse(PeriodIndex==-1, "", substring(temp, PeriodIndex+1))
#    temp <- paste("<A HREF=\"http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=", Org, "&CID=", Number, "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT Gene Symbol.
  indic <- (ColNames %in% c("Gene symbol", "Symbol"))
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
    temp <- paste("<A HREF=\"http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=search&db=gene&term=", temp, "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## HYPERTEXT Ensembl ID.
  indic <- (ColNames == "Ensembl")
  if (sum(indic) > 0) {
    temp <- GeneId[ , (1:NumGeneIds)[indic],drop=F ]
    temp <- paste("<A HREF=\"http://www.ncbi.nlm.nih.gov/gene/?term=", temp, "\" TARGET=_blank>", temp, "</A>", sep="")
    GeneId[ , (1:NumGeneIds)[indic] ] <- temp
  }

  ## CREATE GENE IDS AS A VECTOR OF STRINGS.
  n <- nrow(GeneId)
  temp <- rep( "", n+1 )  #Each row in vector will contain HTML code for one row in resulting table, plus header row.

  # Create HTML code for row numbers.
  if (!missing(ColumnNum)) ColumnNum <- ColumnNum + 1
  GeneId <- if (is.null(GeneRowNum)) {
              cbind( paste("<TR><TD>", 1:n), GeneId )
            } else {
              cbind( paste("<TR><TD>", GeneRowNum), GeneId )
            }

  # Create HTML code for header row.
  # force unwrap in "t-value"
  temp[1] <- ""
  for(i in 1:length(ColNames))
      temp[1] <- paste(temp[1], ifelse(ColNames[i]=="t-value", "<TH NOWRAP>", "<TH>"), ColNames[i])
  temp[1] <- paste("<TR><TH>&nbsp;", temp[1], collapse="") #Qian 6/1/09
  NGeneIds <- ncol(GeneId)
  if (missing(ColumnNum)) {
     for (i in 1:n)
       temp[i+1] <- paste(paste(GeneId[i,], collapse="<BR><TD>"),"<BR>",sep="")
  } else {
     for (i in 1:n) {
       temp[i+1] <- paste(paste(GeneId[i,1:(ColumnNum-1)], collapse="<TD>"),
                    paste(GeneId[i,ColumnNum:NGeneIds], collapse="<TD NOWRAP ALIGN=\"RIGHT\">"),
                    sep="<TD NOWRAP ALIGN=\"RIGHT\">")
    }
  }

  ## GENERATE THE HTML CODE.

  GeneTableHtmlCode <- ""
  ChromDistHtmlCode <- ""

  GeneTableHtmlCode <- c(
      ifelse(length(grep("sorted and grouped", TableCaption)) > 0 #Qian 6/2/09 sortable
    , "<table border='1'>" #Qian 6/2/09 sortable
    , "<table border='1' class='sortable'>") #Qian 6/2/09 sortable
    , "<CAPTION style='text-align:left'>" #use style='text-align:left' to replace ALIGN=Left because the former is both supported in IE and Mozilla Firefox.
    , TableCaption
    , "</CAPTION>"
    , temp
    , "</TABLE>"
  )

  if (RunChromDistBarPlot){
    if (file.access(ChromDistBarPlotFile, mode=4)==0) {  #If file is read-accessible.
      ChromDistHtmlCode <- c("<IMG SRC=\"", ChromDistFileName, "\">")
    }
  }


 ## EITHER RETURN THE HTML CODE, OR WRITE THE HTML CODE TO A FILE.

  if (ReturnHtmlCode) {

    return(
      list(GeneTableHtmlCode = GeneTableHtmlCode
        , GOTableHtmlCode = OEhtml
        , ChromDistHtmlCode = ChromDistHtmlCode
        , DoAnnotations = DoAnnotations
      )
    )

  } else {

    cat("<!-- This is an HTML document.  Please make sure the filename has either"
      , "     an .htm or .html extension, and view this file in a web browser. -->"
      , ""
      , "<HTML>"
      , "<HEAD>"
      , "<STYLE>"
      , "P {font-size: 9pt}"
      , "TABLE {font-size: 9pt}"
      , "</STYLE>"
      , paste("<script src='file:///", ArrayToolsPath, "/misc/sorttable.js'></script>", sep="") #Qian 5/29/09 sortable
      , "</HEAD>"
      , "<BODY>"
      ," ", file=GeneTableFile, sep="\n", append=F
    )
    HeaderStringsVec <- paste(HeaderStringsVec, collapse="<BR>")
    cat(HeaderStringsVec, "<BR><BR>", file=GeneTableFile, sep="\n", append=T)
    cat( "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
      , "<BR><B><U>Gene table:</U></B><BR><BR>"
      , GeneTableHtmlCode
      , file=GeneTableFile, sep="\n", append=T
    )
    if (RunGOOvE) {
      cat( "<P></P>"
        , "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
        , "<P></P><B><U>"
        , paste("'Observed v. Expected' table of GO classes and parent classes,",
            "in list of", nrow(GeneTable), "genes shown above:")
        , "</U></B><P></P>"
        , OEhtml
        , file=GeneTableFile, sep="\n", append=T
      )
    }
    if (RunChromDistBarPlot){
      if (file.access(ChromDistBarPlotFile, mode=4)==0) {  #If file is read-accessible.
        cat( "<P></P>"
          , "<HR SIZE=5 WIDTH=\"100%\" NOSHADE>"
          , "<P></P>"
          , ChromDistHtmlCode
          , file=GeneTableFile, sep="\n", append=T
        )
      }
    }
    cat( "<P></P>"
      , "</BODY>"
      , "</HTML>"
      , file=GeneTableFile, sep="\n", append=T
    )
  }

}
