# Revision history
#
#Qian 8/30/2012 For paired analyses show Patient ID on heatmap for sample labels instead of array ID.
# 2/14/11, Ming-chung add 'xtable' package for global test.
# 4/24/07, Yong implemented Goeman's globaltest for gene set test.
# 12/26/07, Yong implemented Efron-Tibshirani's GSA maxmean test for gene set test.

################################################################################
## The CreateExprSetData function prepares an exprSet class object.
## input:
## arr1: Filtered logged and normalized gene expression matrix for current analysis
## pheno: response variable
## experiment: array or sample or expriment id
## filepath: path for storing binary file
## output:
## create and save an exprSet object: aExprSet to disk
################################################################################

################################################################################
## This function conducts Goeman's GlobalTest for Gene Sets.
## input:
## 1.alpha: p-value cutoff default 0.005
## 2.gtSurv: survival data used when TRUE (default FALSE)
## 3.filepath: path for R binary files
################################################################################
ConductGlobalTest <- function(alpha = 0.005, gtSurv=FALSE, filepath) {

  source(paste(ArrayToolsPath,"/R/BiocUtil.R", sep=""), echo = FALSE)
  print("Loading R package: globaltest ...")
  if (!LoadRPackage("xtable")) return(FALSE)
  if (!LoadRPackage("globaltest", isBioconductor=T)) return(FALSE)

  #read exprSet object from disk
  print("Loading data and gene sets ...")
  load(file=paste(filepath, "/exprSet.rda", sep=""))

  load(file=paste(filepath, "/GeneSetList.rda", sep=""))
  GeneSets <- strsplit(GeneSetList, " ")

  #conduct global test for survival gene sets
  print(paste("Conducting multiple global testing for", length(GeneSets), "gene sets ..."))
  if (gtSurv) {
    # Modify for R 2.10.0
    # gtResult <- globaltest(aExprSet, "Surv(TIMEsurvival, EVENTdeath)", GeneSets)
    require(survival)
    Y <- Surv(aExprSet@phenoData$TIMEsurvival,
              aExprSet@phenoData$EVENTdeath)
    # XX <- t(exprs(aExprSet))
    gtResult <- gt(Y, t(exprs(aExprSet)), subset=GeneSets)
  } else {
    # gtResult <- globaltest(aExprSet, "CLASS", GeneSets)
    Y <- aExprSet@phenoData$CLASS
    # XX <- t(exprs(aExprSet))
    gtResult <- gt(Y, t(exprs(aExprSet)), subset=GeneSets)
  }

  #get info for all genesets
  pValue <- p.value(gtResult)
  nGenes <- unlist(lapply(gtResult@subsets, function(x) length(x)))

  #FDR-adjusted p-value
  pFDR <- p.adjust(pValue, method="fdr")

#  #select by alpha level
#  GeneSetIndex <- which(pValue < alpha)
#  print(paste("There are", length(GeneSetIndex), "gene sets passed the threshold of", alpha))
#
#  #return only significant gene sets
#  nGenes <- nGenes[GeneSetIndex]
#  pValue <- pValue[GeneSetIndex]
#  pFDR <- pFDR[GeneSetIndex];

  #Qihao modified it on 3/16/2010
  GeneSetIndex <- c(1:length(pValue))

  return (data.frame(Index=GeneSetIndex, nGenesInGeneSet=nGenes, pValue=pValue, pFDR=pFDR))
}


################################################################################
## This function conducts Efron-Tibshirani's GSA maxmean method.
## input:
## 1.alpha: p-value cutoff default 0.005
## 2.gsaSurv: survival data used when TRUE (default FALSE)
## 3.gsaPaired: paired data used when TRUE (default FALSE)
## 4.gsanperm: default number of permutations 1000
## 5.mingenes: minimum number of genes in a set default 5
## 6.maxgenes: maximum number of genes in a set default 5000
## 7.filepath: path for R binary files
################################################################################


rowVars <- function(x, na.rm=FALSE, dims=1, unbiased=TRUE, SumSquares=FALSE,
                    twopass=FALSE) {
  if (SumSquares) return(rowSums(x^2, na.rm, dims))
  N <- rowSums(!is.na(x), FALSE, dims)
  Nm1 <- if (unbiased) N-1 else N
  if (twopass) {x <- if (dims==0) x - mean(x, na.rm=na.rm) else
                     sweep(x, 1:dims, rowMeans(x,na.rm,dims))}
  (rowSums(x^2, na.rm, dims) - rowSums(x, na.rm, dims)^2/N) / Nm1
}

flik<- function(y,p)  {
# log liklihood for a*b*x from an F distribution with m and 2*a degrees of freedom
# y is a vector containing data and the m values, p contains a and b.
    x <- y[1:(length(y)/2)]
    m <- y[(length(y)/2+1):length(y)]
    p <- abs(p)
    a <- p[1]
    b <- p[2]
    if(min(a,b)<=0)
      cat ("a,b values were negative there may be a problem\n")
    x <- x*(a*b)
    n <- 2*a
    out <- log(df(x,m,n))+log(a*b)
    sum(-out)
}

getab <- function(sig,n){
    set <- (!is.na(sig)&n>0&sig>0)
    sig <- sig[set]
    n <- n[set]
    set <- n>4
    if (sum(set)>0)  {
        m1 <- (n[set]-2)/((n[set])*sig[set])
        m2 <- (n[set]-2)*(n[set]-4)/((n[set])*sig[set])^2
        m1 <- mean(m1,na.rm=TRUE)
        m2 <- mean(m2,na.rm=TRUE)
        b <- m2/m1-m1
        a <- m1^2/(m2-m1^2)
        # Add the following condition to match with the Fortran program. May 2006.
        if (abs(a) > 10) a <- b <- 1
    }   else{ a <- b <- 1}
    strt <- c(a,b)
    a <- nlm(flik, strt, y=c(sig,n))$estimate
    abs(a)
}

Vaf <- function(x,label, RVM)  {
# RVM model applied to F test (one-way anova), assume equal variance.
# Depends: getab, flik
    lab <- unique(label)
    gnum <- dim(x)[1]
    clnum <- length(lab)
    mni <- ni <- vri <- matrix(nr=gnum, nc=clnum)
    # if (length(lab)==2 & any(table(label) == 1)) return(rep(NA, nrow(x)))
    for(i in 1:clnum)
    {
        mni[,i] <- rowMeans(x[,label==lab[i], drop = FALSE],na.rm=TRUE)
        vri[,i] <- rowVars(x[,label==lab[i], drop = FALSE],na.rm=TRUE)
        ni[,i] <- rowSums(!is.na(x[,label==lab[i], drop = FALSE]),na.rm=TRUE)
    }
    n <- rowSums(ni)
    vr <- rowVars(x,na.rm=TRUE)
    residssq <- rowSums((ni-1)*vri, na.rm = TRUE) # residual S.S.
    totssq <- vr*(n-1)
    num <- (totssq-residssq)/(clnum-1)
    resdf <- n-clnum
    denom <- (residssq)/(resdf) # residual mean squares; residual var estimate.
    Fval <- num/denom
    Fp <- 1-pf(Fval,clnum-1,resdf)
    if (RVM) {
      a <- getab(denom,resdf)
      b <- a[2]
      a <- a[1]
      denom2 <- ((resdf)*denom+2/b)/(resdf+2*a)
      resdf2 <- resdf+2*a
      VFval <- num/denom2
      VFp <- 1-pf(VFval,clnum-1,resdf2)
      return(VFp)
    } else {
      return(Fp)
    }
}

t.testv <- function(x, m1=ncol(x), m2=0, var.equal=TRUE, RVM=FALSE) {
  # T-test by vectorization for two sample or one sample situation.
  # Missing values are denoted by NA and will be taken into account on the
  # degree of freedoms of the T-test.
  # depends on rowVars().
  #
  # The RVM function is specific for one-sample T-test use only (ConductLSgroupcomparison function).
  # So there is no correspoinding part for 2-sample T-test. Two-sample case should be handled in Vaf function.

  m <- m1+m2
  if (nrow(x) > 1)
    x <- as.matrix(x) # Ting 7/23
  if(!is.matrix(x)) x <- matrix(x, nr=1)

  if(ncol(x) != m) stop("m1 or m2 is not specified correctly")
  if(any(is.infinite(m))) stop("Some of expressions are +(-) Inf")
  pval <- rep(NA,nrow(x))
  if (m1<m) {
    # two sample t test
    n1 <- rowSums(!is.na(x[,1:m1,drop=FALSE]))
    n2 <- rowSums(!is.na(x[,(m1+1):m,drop=FALSE]))
    n <- n1+n2
    mean1 <- rowMeans(x[,1:m1,drop=FALSE],na.rm=TRUE)
    mean2 <- rowMeans(x[,(m1+1):m,drop=FALSE],na.rm=TRUE)
    if (var.equal) {
       # denom=pooled variance.
       denom <- (rowVars(x[,1:m1,drop=FALSE], na.rm=TRUE)*(n1-1) +
                 rowVars(x[,(m1+1):m,drop=FALSE], na.rm=TRUE)*(n2-1))/(n-2)
       tmp.sd <- sqrt(denom*((n1+n2)/(n1*n2)))
       tmp.df <- n-2
    }  else {
       # Behren's-Fisher problem
       denom <- rowVars(x[,1:m1,drop=FALSE], na.rm=TRUE)/n1 +
                rowVars(x[,(m1+1):m,drop=FALSE], na.rm=TRUE)/n2
       tmp.sd <- sqrt(denom)
       tmp.df <- denom^2 / ((rowVars(x[,1:m1,drop=FALSE], na.rm=TRUE))^2/(n1^2*(n1-1)) +
                        (rowVars(x[,(m1+1):m,drop=FALSE], na.rm=TRUE))^2/(n2^2*(n2-1)))
    }
    tpt <- (mean1-mean2)/tmp.sd
    ind <- which(n>2)
    # the following way may result in different answer.
    # pval[ind] <- ifelse(tpt[ind]<=0, 2*pt(tpt[ind],tmp.df[ind]), 2*(1-pt(tpt[ind],tmp.df[ind])))
    pval[ind] <- 2 * (1 - pt(abs(tpt[ind]), tmp.df[ind]))
    result <- data.frame(pval=pval, t=tpt, mean=mean1-mean2, sd=tmp.sd, n=n, df=tmp.df)
  }
  else {
    # one sample t test
    n <- apply(x,1,function(x) sum(is.finite(x)))
    tmp.mean <- apply(x,1,mean,na.rm=TRUE)
    tmp.sd <- apply(x,1,sd,na.rm=TRUE)
    tmp <- which(n>1)
	# Added for v4.3 paired data
	if (RVM) {
	  denom <- tmp.sd^2
	  resdf <- n-1
	  a <- getab(denom[tmp], resdf[tmp]) # check line 143 of find_ab.f90
  	  b <- a[2]
	  a <- a[1]
	  denom2 <- ((resdf)*denom+2/b)/(n*(resdf+2*a))  # check line 221 of classcomparion_tests.f90
      resdf2 <- resdf + 2*a
      tmp.t <- tmp.mean/sqrt(denom2)
      pval[tmp] <- 2 * (1 - pt(abs(tmp.t[tmp]), resdf2[tmp])) # check line 224 of classcomparison_tests.f90
	} else {
      tmp.t <- tmp.mean*sqrt(n)/tmp.sd # t-statistic
      pval[tmp] <- 2 * (1 - pt(abs(tmp.t[tmp]), n[tmp] - 1))
	}
    result <- data.frame(pval=pval,t=tmp.t, mean=tmp.mean,sd=tmp.sd,n=n)
  }
  if (is.null(rownames(x)))
    rownames(result) <- paste("v",1:nrow(x), sep="")
  else
    rownames(result) <- rownames(x)
  result
}

ConductLSgroupcomparison <- function(B=1000, RVM=FALSE, filepath, pvalthresh=1e-10, FilterAnalysisSet) {
  #Perform LS test for comparing two groups of arrays. Arrays in each group have class information.
  # Input:
  #   pvalthresh: threshold for the pvalue for individual genes
  #               It is used to prevent sum(-log(p))=Infinity.
  print("Loading data and gene sets ...")
  load(file=paste(filepath, "/GSEAgroupcomparison.rda", sep="")) # y, grp, x, ArrayIds
  grplbl <- unique(grp)
  ylbl <- unique(y)
  if(any(table(y, grp)==1)) {
    cat("Some combination of group and class labels contains only 1 array", file=stderr())
    return(NULL)
  }
  load(file=paste(filepath, "/GeneSetList.rda", sep=""))
  GeneSets <- strsplit(GeneSetList, " ")
  GeneSets <- lapply(GeneSets, as.integer)
  GeneSetIndex <- 1:length(GeneSets)
  nGenes <- sapply(GeneSets, length)

  print(paste("Conducting LS test of comparing two groups for", length(GeneSets), "gene sets ..."))
  if (RVM) {
    # We won't check the RVM assumption since there are 2 groups.
    arr1 <- arr1[, -ncol(arr1)]
	if (!IsPaired) {
      pi.all <- Vaf(arr1[, grp==grplbl[1]], y[grp==grplbl[1]], RVM=TRUE)[FilterAnalysisSet==1]
      qi.all <- Vaf(arr1[, grp==grplbl[2]], y[grp==grplbl[2]], RVM=TRUE)[FilterAnalysisSet==1]
	} else {
	  xtmp <- arr1[, grp==grplbl[1] & y == ylbl[1]] - arr1[, grp==grplbl[1] & y == ylbl[2]]
	  pi.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=TRUE)$pval[FilterAnalysisSet==1]
      xtmp <- arr1[, grp==grplbl[2] & y == ylbl[1]] - arr1[, grp==grplbl[2] & y == ylbl[2]]
	  qi.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=TRUE)$pval[FilterAnalysisSet==1]
    }
  } else {
    if (!IsPaired) {
      pi.all <- Vaf(x[, grp==grplbl[1]], y[grp==grplbl[1]], RVM=FALSE)
      qi.all <- Vaf(x[, grp==grplbl[2]], y[grp==grplbl[2]], RVM=FALSE)
	} else{
	  xtmp <- x[, grp==grplbl[1] & y == ylbl[1]] - x[, grp==grplbl[1] & y == ylbl[2]]
	  pi.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=FALSE)$pval
      xtmp <- x[, grp==grplbl[2] & y == ylbl[1]] - x[, grp==grplbl[2] & y == ylbl[2]]
	  qi.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=FALSE)$pval
    }
  }
  pi.all[pi.all < pvalthresh] <- pvalthresh
  qi.all[qi.all < pvalthresh] <- pvalthresh
  ngeneset <- length(GeneSets)
  test.stat <- rep(NA, ngeneset)
  test.stat.perm <- matrix(NA, nr=ngeneset, nc=B)
  pValue <- rep(1, ngeneset)
  set.seed(1234)
  for(b in 1:B) {
    if (b %% 100 ==0) {
      cat(b, "permutations are done\n")
      cat(paste(b, "permutations are done\n"), file=stderr())
    }
	if (!IsPaired) {
      grp2 <- sample(grp)
	} else {
	  # same id should enjoy the same group label.
      grp2 <- sample(grp[1:(length(grp)/2)]) # use the property that arrays are sorted by (pheno, id)
	  grp2 <- c(grp2, grp2)
    }
    if (all(table(y, grp2) > 1)) {
     # the above condition is for sample t-test case
	 # Can if() statement below be improved?
      if (RVM) {
	    if (!IsPaired) {
          pi.perm <- Vaf(arr1[, grp2==grplbl[1]], y[grp2==grplbl[1]], RVM=TRUE)[FilterAnalysisSet==1]
          qi.perm <- Vaf(arr1[, grp2==grplbl[2]], y[grp2==grplbl[2]], RVM=TRUE)[FilterAnalysisSet==1]
		} else {
	      xtmp <- arr1[, grp2==grplbl[1] & y == ylbl[1]] - arr1[, grp2==grplbl[1] & y == ylbl[2]]
	      pi.perm <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=TRUE)$pval[FilterAnalysisSet==1]
          xtmp <- arr1[, grp2==grplbl[2] & y == ylbl[1]] - arr1[, grp2==grplbl[2] & y == ylbl[2]]
	      qi.perm <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=TRUE)$pval[FilterAnalysisSet==1]
        }
      } else {
	    if (!IsPaired) {
          pi.perm <- Vaf(x[, grp2==grplbl[1]], y[grp2==grplbl[1]], RVM=FALSE)
          qi.perm <- Vaf(x[, grp2==grplbl[2]], y[grp2==grplbl[2]], RVM=FALSE)
		} else {
	      xtmp <- x[, grp2==grplbl[1] & y == ylbl[1]] - x[, grp2==grplbl[1] & y == ylbl[2]]
	      pi.perm <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=FALSE)$pval
          xtmp <- x[, grp2==grplbl[2] & y == ylbl[1]] - x[, grp2==grplbl[2] & y == ylbl[2]]
	      qi.perm <- t.testv(xtmp, m1=ncol(xtmp), m2=0, RVM=FALSE)$pval
        }
      }
      pi.perm[pi.perm < pvalthresh] <- pvalthresh
      qi.perm[qi.perm < pvalthresh] <- pvalthresh
    } else {
      pi.perm <- qi.perm <- rep(NA, sum(FilterAnalysisSet==1))
    }
    for(k in 1:ngeneset) {
        # p[i] is the measure of de between classes for each gene i for group 1
        # q[i] ... for group 2
        # ps[k] is the summary statistic for gene set k in group 1
        #       ps[k] = sum -log(p[i])
        # qs[k] .. for group 2
        # H0: two groups are equivalent wrt the genes in set k
        # Test statistic: ps[k] - qs[k].
        # Permute the group labels and keep the class labels unchanged.
        # Two sided permutation p-value is computed.
        pi <- pi.perm[GeneSets[[k]]]
        qi <- qi.perm[GeneSets[[k]]]
        ps.perm <- sum(-log(pi))
        qs.perm <- sum(-log(qi))
        test.stat.perm[k, b] <- ps.perm - qs.perm
    } # end of for(k in 1:ngeneset)
  } # end of for(b in 1:B)
  for(k in 1:ngeneset) {
    # if (k %% 100 ==0) {
    #   cat(k, "gene sets are done\n")
    #   cat(paste(k, "gene sets are done\n"), file=stderr())
    # }
    pi <- pi.all[GeneSets[[k]]] # GeneSets is relative to the filtered genes.
    qi <- qi.all[GeneSets[[k]]]
    ps.obs <- sum(-log(pi)) # can be Inf, -Inf or a finite number
    qs.obs <- sum(-log(qi))
    test.stat[k] <- ps.obs - qs.obs #  can be NA, Inf, -Inf or a finite number
    pValue[k] <- ifelse(test.stat[k] >= 0,
                    2*((1+sum(1*(test.stat.perm[k,] >= test.stat[k]), na.rm=TRUE))/(1+sum(is.finite(test.stat.perm[k,])))),
                    2*((1+sum(1*(test.stat.perm[k,] <= test.stat[k]), na.rm=TRUE))/(1+sum(is.finite(test.stat.perm[k,])))))
  }
  pValue <- pmin(pValue, 1)

  return(list(res=data.frame(Index=GeneSetIndex, nGenesInGeneSet=nGenes, pValue=pValue),
              test.stat=test.stat, test.stat.perm=test.stat.perm))
}

ConductMaxmeangroupcomparison <- function(B=1000, filepath) {
  #Perform maxmean test for comparing two groups of arrays. Arrays in each group have class information.
  print("Loading data and gene sets ...")
  load(file=paste(filepath, "/GSEAgroupcomparison.rda", sep="")) # y, grp, x, ArrayIds
  grplbl <- unique(grp)
  if(any(table(y, grp)==1)) {
    cat("Some combination of group and class labels contains only 1 array", file=stderr())
    return(NULL)
  }
  load(file=paste(filepath, "/GeneSetList.rda", sep=""))
  GeneSets <- strsplit(GeneSetList, " ")
  GeneSets <- lapply(GeneSets, as.integer)
  GeneSetIndex <- 1:length(GeneSets)
  nGenes <- sapply(GeneSets, length)

  print(paste("Conducting Maxmean test of comparing two groups for", length(GeneSets), "gene sets ..."))
  ngene <- nrow(x)

  uy <- unique(y)
  # For group p
  if (!IsPaired) {
    xtmp <- cbind(x[, grp==grplbl[1] & y==uy[1]], x[, grp==grplbl[1] & y==uy[2]])
    zi.p.all <- t.testv(xtmp, sum(grp==grplbl[1] & y==uy[1]),
                          sum(grp==grplbl[1] & y==uy[2]))$t
  } else {
    xtmp <- x[, grp==grplbl[1] & y == uy[1]] - x[, grp==grplbl[1] & y == uy[2]]
	zi.p.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0)$t
  }
  s.pos.p <- pmax(zi.p.all, 0)
  s.neg.p <- -pmin(zi.p.all, 0)
  mean.p <- c(pos=mean(s.pos.p, na.rm=TRUE), neg=mean(s.neg.p, na.rm=TRUE))
  sd.p <- c(pos=sd(s.pos.p, na.rm=TRUE), neg=sd(s.neg.p, na.rm=TRUE))
  # For group q
  if (!IsPaired) {
    xtmp <- cbind(x[, grp==grplbl[2] & y==uy[1]], x[, grp==grplbl[2] & y==uy[2]])
    zi.q.all <- t.testv(xtmp, sum(grp==grplbl[2] & y==uy[1]),
                          sum(grp==grplbl[2] & y==uy[2]))$t
  } else {
    xtmp <- x[, grp==grplbl[2] & y == uy[1]] - x[, grp==grplbl[2] & y == uy[2]]
	zi.q.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0)$t
  }
  s.pos.q <- pmax(zi.q.all, 0) # vector
  s.neg.q <- -pmin(zi.q.all, 0) # vector
  mean.q <- c(pos=mean(s.pos.q, na.rm=TRUE), neg=mean(s.neg.q, na.rm=TRUE))
  sd.q <- c(pos=sd(s.pos.q, na.rm=TRUE), neg=sd(s.neg.q, na.rm=TRUE))

  s.pos.perm.p <- s.neg.perm.p <- s.pos.perm.q <- s.neg.perm.q <- matrix(NA, nr=ngene, nc=B)
  mean.perm.p <- sd.perm.p <- mean.perm.q <- sd.perm.q <- matrix(NA, nr=2, nc=B)
  rownames(mean.perm.p) <- rownames(sd.perm.p) <- rownames(mean.perm.q) <- rownames(sd.perm.q) <- c("pos", "neg")
  ngeneset <- length(GeneSets)
  test.stat <- rep(NA, ngeneset)
  test.stat.perm <- matrix(NA, nr=ngeneset, nc=B)
  pValue <- rep(1, ngeneset)
  set.seed(1234)
  for(b in 1:B) {
    if (b %% 100 ==0) {
      cat(b, "permutations are done\n")
      cat(paste(b, "permutations are done\n"), file=stderr())
    }
	if (!IsPaired) {
      grp2 <- sample(grp)
	} else {
	  # same id should enjoy the same group label.
      grp2 <- sample(grp[1:(length(grp)/2)]) # use the property that arrays are sorted by (pheno, id)
	  grp2 <- c(grp2, grp2)
    }
    if (all(table(y, grp2) > 1)) {
      # the above condition is for 2-sample t-test case or paired data
	  # For group p
	  if (!IsPaired) {
        xtmp <- cbind(x[, grp2==grplbl[1] & y==uy[1]], x[, grp2==grplbl[1] & y==uy[2]])
        zi.p.all <- t.testv(xtmp, sum(grp2==grplbl[1] & y==uy[1]),
                                sum(grp2==grplbl[1] & y==uy[2]))$t
      } else {
        xtmp <- x[, grp2==grplbl[1] & y == uy[1]] - x[, grp2==grplbl[1] & y == uy[2]]
	    zi.p.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0)$t
      }
      s.pos.perm.p <- pmax(zi.p.all, 0)
      s.neg.perm.p <- -pmin(zi.p.all, 0)
      mean.perm.p <- c(pos=mean(s.pos.perm.p, na.rm=TRUE), neg=mean(s.neg.perm.p, na.rm=TRUE))
      sd.perm.p <- c(pos=sd(s.pos.perm.p, na.rm=TRUE), neg=sd(s.neg.perm.p, na.rm=TRUE))
      # For group q
	  if (!IsPaired) {
        xtmp <- cbind(x[, grp2==grplbl[2] & y==uy[1]], x[, grp2==grplbl[2] & y==uy[2]])
        zi.q.all <- t.testv(xtmp, sum(grp2==grplbl[2] & y==uy[1]),
                              sum(grp2==grplbl[2] & y==uy[2]))$t
      } else {
        xtmp <- x[, grp2==grplbl[2] & y == uy[1]] - x[, grp2==grplbl[2] & y == uy[2]]
	    zi.q.all <- t.testv(xtmp, m1=ncol(xtmp), m2=0)$t
      }
      s.pos.perm.q <- pmax(zi.q.all, 0)
      s.neg.perm.q <- -pmin(zi.q.all, 0)
      mean.perm.q <- c(pos=mean(s.pos.perm.q, na.rm=TRUE), neg=mean(s.neg.perm.q, na.rm=TRUE))
      sd.perm.q <- c(pos=sd(s.pos.perm.q, na.rm=TRUE), neg=sd(s.neg.perm.q, na.rm=TRUE))
    }
    gsa.stat.perm.p <- function(index, mean.perm.p, sd.perm.p) {
      sbar.pos <- mean(s.pos.perm.p[index], na.rm=TRUE)
      sbar.neg <- mean(s.neg.perm.p[index], na.rm=TRUE)
      maxmean.stat <- max((sbar.pos - mean.perm.p["pos"])/sd.perm.p["pos"],
                          (sbar.neg - mean.perm.p["neg"])/sd.perm.p["neg"])
    }
    gsa.stat.perm.q <- function(index, mean.perm.q, sd.perm.q) {
      sbar.pos <- mean(s.pos.perm.q[index], na.rm=TRUE)
      sbar.neg <- mean(s.neg.perm.q[index], na.rm=TRUE)
      maxmean.stat <- max((sbar.pos - mean.perm.q["pos"])/sd.perm.q["pos"],
                          (sbar.neg - mean.perm.q["neg"])/sd.perm.q["neg"])
    }
    for(k in 1:ngeneset) {
      ps.perm <- gsa.stat.perm.p(GeneSets[[k]], mean.perm.p, sd.perm.p)
      qs.perm <- gsa.stat.perm.q(GeneSets[[k]], mean.perm.q, sd.perm.q)
      test.stat.perm[k, b] <- ps.perm - qs.perm
    }
  } # end of for(b in 1:B)
  gsa.stat.p <- function(index, mean.p, sd.p) {
    # Global variables
    #   s.pos.p: max(zi, 0) for all genes
    #   s.neg.p: -min(zi, 0) for all genes
    #   mean.pos.p, (mean.neg.p): mean of s.pos.p (s.neg.p)
    #   sd.pos.p, (sd.neg.p): sd of s.pos.p (s.neg.p)
    # Argument:
    #   index: index of gene set
    # Output:
    #   GSA test statistic
    sbar.pos <- mean(s.pos.p[index], na.rm=TRUE)
    sbar.neg <- mean(s.neg.p[index], na.rm=TRUE)
    maxmean.stat <- max((sbar.pos - mean.p["pos"])/sd.p["pos"], (sbar.neg - mean.p["neg"])/sd.p["neg"])
  }
  gsa.stat.q <- function(index, mean.q, sd.q) {
    sbar.pos <- mean(s.pos.q[index], na.rm=TRUE)
    sbar.neg <- mean(s.neg.q[index], na.rm=TRUE)
    maxmean.stat <- max((sbar.pos - mean.q["pos"])/sd.q["pos"], (sbar.neg - mean.q["neg"])/sd.q["neg"])
  }
  for(k in 1:ngeneset) {
      # gsa.stat.p[k] is the summary statistic for gene set k based on maxmean statistic
      #               for arrays in group 1
      # gsa.stat.q[k] .. for group 2
      # H0: two groups are equivalent wrt the genes in set k
      # Test statistic: gsa.stat.p[k] - gsa.stat.q[k].
      # Permute the group labels and keep the class labels unchanged.
      # Two sided permutation p-value is computed.
      # if (k %% 100 ==0) {
      #   cat(k, "gene sets are done\n")
      #   cat(paste(k, "gene sets are done\n"), file=stderr())
      # }
      ps.obs <- gsa.stat.p(GeneSets[[k]], mean.p, sd.p)
      qs.obs <- gsa.stat.q(GeneSets[[k]], mean.q, sd.q)
      test.stat[k] <- ps.obs - qs.obs
      pValue[k] <- ifelse(test.stat[k] >= 0,
                        2*((1+sum(1*(test.stat.perm[k,] >= test.stat[k]), na.rm=TRUE))/(1+sum(is.finite(test.stat.perm[k,])))),
                        2*((1+sum(1*(test.stat.perm[k,] <= test.stat[k]), na.rm=TRUE))/(1+sum(is.finite(test.stat.perm[k,])))))
  }
  pValue <- pmin(pValue, 1)

  return(list(res=data.frame(Index=GeneSetIndex, nGenesInGeneSet=nGenes, pValue=pValue),
              test.stat=test.stat, test.stat.perm=test.stat.perm))
}



#Helper function for showing gene set specific heatmap
GeneSetHeatmapHtml <- function(outputpath, n) {

  filename1 <- paste(outputpath,"geneset1",sep="/")
  filename1n <- "geneset1"
  filename2n <- "geneset2"

  cat("<html> <head> <title>Gene Set 1 Heatmap</title></head>","\n",file=paste(filename1,".html",sep=""))
  cat("<body>","\n",file=paste(filename1,".html",sep=""),append=TRUE)
  if (n>=2) cat(paste("<p><a href=",paste("\"",filename2n,".html\"",sep="")),"><font color=\"blue\">Next</font></a></p>\n",file=paste(filename1,".html",sep=""),append=TRUE)
  cat(paste("<img src=",paste("\"",filename1n,".png\"",sep="")),"><br />\n",file=paste(filename1,".html",sep=""),append=TRUE)
  cat("To modify the heatmap colors and threshold parameters, from the \"ArrayTools\" pull down menu select Utilities -> Preference and then edit the corresponding settings.\n",file=paste(filename1,".html",sep=""),append=TRUE)
  cat("</body> </html>\n",file=paste(filename1,".html",sep=""),append=TRUE)
  if (n >=2) {
    for(j in 2:n) {
      filename0n <- paste("geneset",j-1,sep="")
      filename1 <- paste(outputpath,paste("geneset",j,sep=""),sep="/")
      filename1n <- paste("geneset",j,sep="")
      filename2n <- paste("geneset",j+1,sep="")

      cat(paste("<html> <head> <title>Gene Set ", j, " Heatmap</title></head>", sep=""),"\n",file=paste(filename1,".html",sep=""))
      cat("<body>","\n",file=paste(filename1,".html",sep=""),append=TRUE)
      if (j != n) {
        cat(paste("<p><a href=",paste("\"",filename0n,".html\"",sep="")),"><font color=\"blue\">Previous</font></a>\n",file=paste(filename1,".html",sep=""),append=TRUE)
        cat(paste("<a href=",paste("\"",filename2n,".html\"",sep="")),"><font color=\"blue\"> Next</font></a></p>\n",file=paste(filename1,".html",sep=""),append=TRUE)
      } else {
        cat(paste("<p><a href=",paste("\"",filename0n,".html\"",sep="")),"><font color=\"blue\">Previous</font></a></p>\n",file=paste(filename1,".html",sep=""),append=TRUE)
      }
      cat(paste("<img src=",paste("\"",filename1n,".png\"",sep="")),"><br />\n",file=paste(filename1,".html",sep=""),append=TRUE)
      cat("To modify the heatmap colors and threshold parameters, from the \"ArrayTools\" pull down menu select Utilities -> Preference and then edit the corresponding settings.\n",file=paste(filename1,".html",sep=""),append=TRUE) #Qian 7/21/09
      cat("</body> </html>\n",file=paste(filename1,".html",sep=""),append=TRUE)
    }
  }
}

